@@ -28,6 +28,7 @@
 #include "sdlapp.h"
 
 std::string gSDLAppResourceDir;
+std::string gSDLAppConfDir;
 
 #ifdef _WIN32
 std::string gSDLAppPathSeparator = "\\";
@@ -35,14 +36,114 @@ std::string gSDLAppPathSeparator = "\\";
 std::string gSDLAppPathSeparator = "/";
 #endif
 
+std::string gSDLAppTitle = "SDL App";
+std::string gSDLAppExec  = "sdlapp";
+
+#ifdef _WIN32
+HWND gSDLAppConsoleWindow = 0;
+
+void SDLAppCreateWindowsConsole() {
+    if(gSDLAppConsoleWindow !=0) return;
+
+    //create a console on Windows so users can see messages
+
+    //find an available name for our window
+    int console_suffix = 0;
+    char consoleTitle[512];
+    sprintf(consoleTitle, "%s Console", gSDLAppTitle.c_str());
+
+    while(FindWindow(0, consoleTitle)) {
+        sprintf(consoleTitle, "%s Console %d", gSDLAppTitle.c_str(), ++console_suffix);
+    }
+
+    AllocConsole();
+    SetConsoleTitle(consoleTitle);
+
+    //redirect streams to console
+    freopen("conin$", "r", stdin);
+    freopen("conout$","w", stdout);
+    freopen("conout$","w", stderr);
+
+    gSDLAppConsoleWindow = 0;
+
+    //wait for our console window
+    while(gSDLAppConsoleWindow==0) {
+        gSDLAppConsoleWindow = FindWindow(0, consoleTitle);
+        SDL_Delay(100);
+    }
+
+    //disable the close button so the user cant crash gource
+    HMENU hm = GetSystemMenu(gSDLAppConsoleWindow, false);
+    DeleteMenu(hm, SC_CLOSE, MF_BYCOMMAND);
+}
+
+void SDLAppResizeWindowsConsole(int height) {
+    if(gSDLAppConsoleWindow !=0) {
+        RECT windowRect;
+        if(GetWindowRect(gSDLAppConsoleWindow, &windowRect)) {
+            float width = windowRect.right - windowRect.left;
+            MoveWindow(gSDLAppConsoleWindow,windowRect.left,windowRect.top,width,height,true);
+        }
+    }
+}
+
+#endif
+
 bool SDLAppDirExists(std::string dir) {
     struct stat st;
     return !stat(dir.c_str(), &st) && S_ISDIR(st.st_mode);
 }
 
-void SDLAppInit() {
-    if(gSDLAppResourceDir.size()>0) return;
+std::string SDLAppAddSlash(std::string path) {
 
+    //append slash unless the path is empty
+    if(path.size() && path[path.size()-1] != gSDLAppPathSeparator[0]) {
+        path += gSDLAppPathSeparator;
+    }
+
+    return path;
+}
+
+//info message
+void SDLAppInfo(std::string msg) {
+#ifdef _WIN32
+    SDLAppCreateWindowsConsole();
+#endif
+
+    printf("%s\n", msg.c_str());
+
+#ifdef _WIN32
+    printf("\nPress Enter\n");
+    getchar();
+#endif
+
+    exit(0);
+}
+
+//display error only
+void SDLAppQuit(std::string error) {
+    SDL_Quit();
+
+#ifdef _WIN32
+    SDLAppCreateWindowsConsole();
+#endif
+
+    fprintf(stderr, "%s: %s\n", gSDLAppExec.c_str(), error.c_str());
+    fprintf(stderr, "Try '%s --help' for more information.\n\n", gSDLAppExec.c_str());
+
+#ifdef _WIN32
+    fprintf(stderr, "Press Enter\n");
+    getchar();
+#endif
+
+    exit(1);
+}
+
+void SDLAppInit(std::string apptitle, std::string execname) {
+    gSDLAppTitle = apptitle;
+    gSDLAppExec  = execname;
+
+    std::string conf_dir     = "";
     std::string resource_dir = "data/";
     std::string fonts_dir    = "data/fonts/";
 #ifdef _WIN32
@@ -55,6 +156,7 @@ void SDLAppInit() {
     int pos = exepath.rfind("\\");
 
     std::string path = exepath.substr(0, pos+1);
+    conf_dir     = path + std::string("\\");
     resource_dir = path + std::string("\\data\\");
     fonts_dir    = path + std::string("\\data\\fonts\\");
 #else
@@ -62,12 +164,19 @@ void SDLAppInit() {
     char cwd_buff[1024];
 
     if(getcwd(cwd_buff, 1024) == cwd_buff) {
+        conf_dir     = std::string(cwd_buff) + std::string("/");
         resource_dir = std::string(cwd_buff) + std::string("/") + resource_dir;
         fonts_dir    = std::string(cwd_buff) + std::string("/") + fonts_dir;
     }
 
 #endif
 
+#ifdef SDLAPP_CONF_DIR
+    if (SDLAppDirExists(SDLAPP_CONF_DIR)) {
+        conf_dir = SDLAPP_CONF_DIR;
+    }
+#endif
+
 #ifdef SDLAPP_RESOURCE_DIR
     if (SDLAppDirExists(SDLAPP_RESOURCE_DIR)) {
         resource_dir = SDLAPP_RESOURCE_DIR;
@@ -81,41 +190,52 @@ void SDLAppInit() {
     }
 #endif
 
+    resource_dir = SDLAppAddSlash(resource_dir);
+    conf_dir     = SDLAppAddSlash(conf_dir);
+    fonts_dir    = SDLAppAddSlash(fonts_dir);
+
     texturemanager.setDir(resource_dir);
     fontmanager.setDir(fonts_dir);
 
     gSDLAppResourceDir = resource_dir;
+    gSDLAppConfDir     = conf_dir;
 }
 
 void SDLAppParseArgs(int argc, char *argv[], int* xres, int* yres, bool* fullscreen, std::vector<std::string>* otherargs) {
-    SDLAppInit();
 
     for (int i=1; i<argc; i++) {
         debugLog("argv[%d] = %s\n", i, argv[i]);
 
-        if (!strcmp(argv[i],"-f")) {
-            *fullscreen = 1;
+        std::string args(argv[i]);
+
+        if (args == "-f") {
+            *fullscreen = true;
             continue;
         }
-        else if (!strcmp(argv[i],"-w")) {
-            *fullscreen = 0;
+        else if (args == "-w") {
+            *fullscreen = false;
             continue;
         }
 
         //get video mode
-        if(strchr(argv[i], '-') != 0 && strlen(argv[i])>1) {
-            std::string argstr(argv[i], 1, strlen(argv[i])-1);
-            debugLog("%s\n", argstr.c_str());
-            size_t x = argstr.rfind("x");
+        if(args.size()>1 && args[0] == '-' && args.rfind("x") != std::string::npos) {
+
+            std::string displayarg = args;
+
+            while(displayarg.size()>1 && displayarg[0] == '-') {
+                displayarg = displayarg.substr(1, displayarg.size()-1);
+            }
+
+            size_t x = displayarg.rfind("x");
 
             if(x != std::string::npos) {
-                std::string widthstr  = argstr.substr(0, x);
-                std::string heightstr = argstr.substr(x+1);
+                std::string widthstr  = displayarg.substr(0, x);
+                std::string heightstr = displayarg.substr(x+1);
 
                 int width = atoi(widthstr.c_str());
                 int height = atoi(heightstr.c_str());
 
-                if(width!=0 && height!=0) {
+                if(width>0 && height>0) {
                     debugLog("w=%d, h=%d\n",width,height);
 
                     *xres = width;
@@ -127,7 +247,7 @@ void SDLAppParseArgs(int argc, char *argv[], int* xres, int* yres, bool* fullscr
 
         // non display argument
         if(otherargs != 0) {
-            otherargs->push_back(std::string(argv[i]));
+            otherargs->push_back(args);
         }
     }
 }
@@ -179,9 +299,9 @@ int SDLApp::run() {
 
         Uint32 delta_msec = msec - last_msec;
 
-        // cant have delta ticks be 0
+        // cant have delta ticks be less than 8ms
         buffer_msec += delta_msec;
-        if(buffer_msec < 1) {
+        if(buffer_msec < 8) {
             SDL_Delay(1);
             continue;
         }
