@@ -29,8 +29,6 @@ class RFile : public Pawn {
     vec3f file_colour;
     vec3f touch_colour;
 
-    int path_hash;
-
     RDirNode* dir;
 
     bool removing;
@@ -43,16 +41,16 @@ class RFile : public Pawn {
     vec2f dest;
     float distance;
 
-    GLuint namelist;
+    //GLuint namelist;
 
-    std::string shortname;
+    void setFilename(const std::string& abs_file_path);
 
-    void setPath();
     const vec3f& getNameColour() const;
     void drawNameText(float alpha) const;
 public:
     std::string path;
     std::string fullpath;
+    std::string ext;
 
     RFile(const std::string & name, const vec3f & colour, const vec2f & pos, int tagid);
     ~RFile();
@@ -60,15 +58,16 @@ public:
     bool isExpiring() const { return expiring; }
     bool isRemoving() const { return removing; }
     
+    bool overlaps(const vec2f& pos) const; 
+    
     const vec3f & getFileColour() const;
     vec3f getColour() const;
+    void colourize();
 
     float getAlpha() const;
 
     void touch(const vec3f & colour);
-
-    const std::string & getFullPath() const;
-
+   
     void setSelected(bool selected);
 
     void setHidden(bool hidden);
@@ -85,8 +84,6 @@ public:
 
     RDirNode* getDir() const;
     void setDir(RDirNode* dir);
-
-    int getPathHash() const;
 };
 
 extern float gGourceFileDiameter;
