@@ -100,10 +100,16 @@ Gource::Gource(FrameExporter* exporter) {
     hoverUser = 0;
 
     date_x_offset = 0;
-
+    
+    textbox = TextBox(fontmanager.grab("FreeSans.ttf", 18));
+    textbox.setBrightness(0.5f);
+    textbox.show();
+    
+    file_key = FileKey(1.0f);
+    
     camera = ZoomCamera(vec3f(0,0, -300), vec3f(0.0, 0.0, 0.0), 250.0, 5000.0);
     camera.setPadding(gGourceSettings.padding);
-    
+
     setCameraMode(gGourceSettings.camera_mode);
 
     root = 0;
@@ -122,6 +128,33 @@ Gource::Gource(FrameExporter* exporter) {
     if(exporter==0 && gGourceSettings.repo_count==1) slider.show();
 }
 
+void Gource::writeCustomLog(const std::string& logfile, const std::string& output_file) {
+    RCommitLog* commitlog = determineFormat(logfile);
+
+    if(!commitlog) return;
+
+    RCommit commit;
+
+    FILE* fh = stdout;
+
+    if(output_file != "-") {
+        fh = fopen(output_file.c_str(), "w");
+
+        if(!fh) return;
+    }
+
+    while(commitlog->nextCommit(commit)) {
+        for(std::list<RCommitFile>::iterator it = commit.files.begin(); it != commit.files.end(); it++) {
+            RCommitFile& cf = *it;
+            fprintf(fh, "%lld|%s|%s|%s\n", (long long int) commit.timestamp, commit.username.c_str(), cf.action.c_str(), cf.filename.c_str());
+        }
+        commit.files.clear();
+    }
+
+    if(output_file != "-") fclose(fh);
+
+}
+
 RCommitLog* Gource::determineFormat(const std::string& logfile) {
     debugLog("determineFormat(%s)\n", logfile.c_str());
 
@@ -170,6 +203,18 @@ RCommitLog* Gource::determineFormat(const std::string& logfile) {
             delete clog;
         }
 
+        if(gGourceSettings.log_format == "svn") {
+            clog = new SVNCommitLog(logfile);
+            if(clog->checkFormat()) return clog;
+            delete clog;
+        }
+
+        if(gGourceSettings.log_format == "cvs2cl") {
+            clog = new CVS2CLCommitLog(logfile);
+            if(clog->checkFormat()) return clog;
+            delete clog;
+        }
+
         return 0;
     }
 
@@ -208,6 +253,20 @@ RCommitLog* Gource::determineFormat(const std::string& logfile) {
 
     delete clog;
 
+    //svn
+    debugLog("trying svn...\n");
+    clog = new SVNCommitLog(logfile);
+    if(clog->checkFormat()) return clog;
+
+    delete clog;
+    
+    //cvs2cl
+    debugLog("trying cvs2cl...\n");
+    clog = new CVS2CLCommitLog(logfile);
+    if(clog->checkFormat()) return clog;
+
+    delete clog;
+    
     //custom
     debugLog("trying custom...\n");
     clog = new CustomLog(logfile);
@@ -248,7 +307,7 @@ void Gource::update(float t, float dt) {
     }
 
     //apply time scaling
-    scaled_dt *= time_scale;
+    scaled_dt *= gGourceSettings.time_scale;
 
     //have to manage runtime internally as we're messing with dt
     if(!paused) runtime += scaled_dt;
@@ -337,7 +396,7 @@ void Gource::mouseMove(SDL_MouseMotionEvent *e) {
             return;
         }
 
-        backgroundPos += mag;
+        cursor_move = mag;
 
         return;
     }
@@ -360,24 +419,23 @@ void Gource::mouseMove(SDL_MouseMotionEvent *e) {
 
 void Gource::zoom(bool zoomin) {
 
-    float min_distance = camera.getMinDistance();
-    float max_distance = camera.getMaxDistance();
+    manual_zoom = true;
 
     float zoom_multi = 1.1;
 
+    float distance = -camera.getDest().z;
+
     if(zoomin) {
-        min_distance /= zoom_multi;
-        if(min_distance < 100.0f)
-            min_distance = 100.0f;
+        distance /= zoom_multi;
 
-        camera.setMinDistance(min_distance);
+        if(distance < 100.0f) distance = 100.0f;
     } else {
-        min_distance *= zoom_multi;
-        if(min_distance > 4999.0f)
-            min_distance = 4999.0f;
+        distance *= zoom_multi;
 
-        camera.setMinDistance(min_distance);
+        if(distance > 4999.0f) distance = 4999.0f;
     }
+
+    camera.setDistance(distance);
 }
 
 void Gource::mouseClick(SDL_MouseButtonEvent *e) {
@@ -472,10 +530,13 @@ void Gource::setCameraMode(const std::string& mode) {
 }
 
 void Gource::setCameraMode(bool track_users) {
-    manual_rotate = false;
+    manual_rotate   = false;
+    manual_zoom     = false;
+
     this->track_users = track_users;
     if(selectedUser!=0) camera.lockOn(track_users);
-    backgroundSelected=false;
+
+    manual_camera = false;
 
     gGourceSettings.camera_mode = track_users ? "track" : "overview";
 }
@@ -492,9 +553,7 @@ void Gource::selectBackground() {
 
     selectUser(0);
 
-    backgroundSelected = true;
-
-    backgroundPos = camera.getPos().truncate();
+    manual_camera = true;
 
     cursor.showCursor(false);
     grab_mouse=true;
@@ -506,8 +565,6 @@ void Gource::selectUser(RUser* user) {
     //already selected do nothing
     if(user!=0 && selectedUser==user) return;
 
-    backgroundSelected=false;
-
     if(selectedFile != 0) {
         selectedFile->setSelected(false);
         selectedFile = 0;
@@ -539,8 +596,6 @@ void Gource::selectFile(RFile* file) {
     //already selected do nothing
     if(file!=0 && selectedFile==file) return;
 
-    backgroundSelected=false;
-
     if(selectedUser != 0) {
         selectedUser->setSelected(false);
         selectedUser = 0;
@@ -651,15 +706,53 @@ void Gource::keyPress(SDL_KeyboardEvent *e) {
         }
 
         if (e->keysym.sym == SDLK_u) {
-            gGourceSettings.hide_usernames = !gGourceSettings.hide_usernames;
+            
+            if(gGourceSettings.hide_usernames && !gGourceSettings.highlight_all_users) {
+                gGourceSettings.hide_usernames      = false;
+                gGourceSettings.highlight_all_users = true;
+
+            } else if (gGourceSettings.highlight_all_users && !gGourceSettings.hide_usernames) {
+                gGourceSettings.hide_usernames      = false;
+                gGourceSettings.highlight_all_users = false;
+            } else {
+                gGourceSettings.hide_usernames      = true;
+                gGourceSettings.highlight_all_users = false;
+            }
+           
         }
 
         if (e->keysym.sym == SDLK_d) {
-            gGourceSettings.hide_dirnames = ! gGourceSettings.hide_dirnames;
+            if(gGourceSettings.hide_dirnames && !gGourceSettings.highlight_dirs) {
+            
+                gGourceSettings.hide_dirnames  = false;
+                gGourceSettings.highlight_dirs = true;
+
+            } else if(gGourceSettings.highlight_dirs && !gGourceSettings.hide_dirnames) {
+
+                gGourceSettings.hide_dirnames  = false;
+                gGourceSettings.highlight_dirs = false;
+
+            } else {
+                gGourceSettings.hide_dirnames  = true;
+                gGourceSettings.highlight_dirs = false;
+            }
         }
 
         if (e->keysym.sym == SDLK_f) {
-            gGourceSettings.hide_filenames = !gGourceSettings.hide_filenames;
+            
+            if(gGourceSettings.hide_filenames && !gGourceSettings.file_extensions) {
+                gGourceSettings.hide_filenames  = false;
+            } else if(!gGourceSettings.hide_filenames && gGourceSettings.file_extensions) {
+                gGourceSettings.file_extensions = false;
+                gGourceSettings.hide_filenames = true;
+            } else {
+                gGourceSettings.file_extensions = true;
+                gGourceSettings.hide_filenames  = false;           
+            }
+        }
+
+        if (e->keysym.sym == SDLK_k) {
+            gGourceSettings.show_key = !gGourceSettings.show_key;
         }
 
         if(e->keysym.sym == SDLK_c) {
@@ -674,6 +767,10 @@ void Gource::keyPress(SDL_KeyboardEvent *e) {
             gGourceGravity = !gGourceGravity;
         }
 
+        if (e->keysym.sym == SDLK_s) {
+            recolour=true;
+        }
+
         if(e->keysym.unicode == SDLK_TAB) {
             selectNextUser();
         }
@@ -698,11 +795,11 @@ void Gource::keyPress(SDL_KeyboardEvent *e) {
             }
         }
 
-        if(e->keysym.sym == SDLK_UP) {
+        if(e->keysym.sym == SDLK_KP_MINUS) {
             zoom(true);
         }
 
-        if(e->keysym.sym == SDLK_DOWN) {
+        if(e->keysym.sym == SDLK_KP_PLUS) {
             zoom(false);
         }
 
@@ -716,24 +813,24 @@ void Gource::keyPress(SDL_KeyboardEvent *e) {
 
         if(e->keysym.unicode == SDLK_PERIOD) {
 
-            if(time_scale>=1.0) {
-                time_scale = std::min(4.0f, floorf(time_scale) + 1.0f);
+            if(gGourceSettings.time_scale>=1.0) {
+                gGourceSettings.time_scale = std::min(4.0f, floorf(gGourceSettings.time_scale) + 1.0f);
             } else {
-                time_scale = std::min(1.0f, time_scale * 2.0f);
+                gGourceSettings.time_scale = std::min(1.0f, gGourceSettings.time_scale * 2.0f);
             }
         }
 
         if(e->keysym.unicode == SDLK_COMMA) {
 
-            if(time_scale>1.0) {
-                time_scale = std::max(0.0f, floorf(time_scale) - 1.0f);
+            if(gGourceSettings.time_scale>1.0) {
+                gGourceSettings.time_scale = std::max(0.0f, floorf(gGourceSettings.time_scale) - 1.0f);
             } else {
-                time_scale = std::max(0.25f, time_scale * 0.5f);
+                gGourceSettings.time_scale = std::max(0.25f, gGourceSettings.time_scale * 0.5f);
             }
         }
 
         if(e->keysym.unicode == SDLK_SLASH) {
-            time_scale = 1.0f;
+            gGourceSettings.time_scale = 1.0f;
         }
     }
 }
@@ -741,6 +838,7 @@ void Gource::keyPress(SDL_KeyboardEvent *e) {
 void Gource::reset() {
     camera.reset();
     user_bounds.reset();
+    active_user_bounds.reset();
     dir_bounds.reset();
     commitqueue.clear();
     tagfilemap.clear();
@@ -750,18 +848,29 @@ void Gource::reset() {
     if(userTree!=0) delete userTree;
     if(dirNodeTree!=0) delete dirNodeTree;
 
+    recolour = false;
+
     userTree = 0;
     dirNodeTree = 0;
 
     selectedFile = 0;
     hoverFile = 0;
 
-    manual_rotate = false;
+    use_selection_bounds = false;
+    selection_bounds.reset();
+    
+    manual_rotate   = false;
+    manual_zoom     = false;
     rotation_remaining_angle = 0.0f;
 
+    message_timer = 0.0f;
+
+    cursor_move = vec2f(0.0f, 0.0f);
+  
     selectedUser = 0;
     hoverUser = 0;
-    backgroundSelected=false;
+
+    manual_camera = false;
 
     grab_mouse = false;
 
@@ -790,8 +899,8 @@ void Gource::reset() {
 
     files.clear();
 
-
-    time_scale = 1.0f;
+    file_key.clear();
+    
     idle_time=0;
     currtime=0;
     lasttime=0;
@@ -801,7 +910,7 @@ void Gource::reset() {
 }
 
 void Gource::deleteFile(RFile* file) {
-    debugLog("removing file %s\n", file->getFullPath().c_str());
+    debugLog("removing file %s\n", file->fullpath.c_str());
 
     root->removeFile(file);
 
@@ -820,10 +929,11 @@ void Gource::deleteFile(RFile* file) {
         user->fileRemoved(file);
     }
 
-    files.erase(file->getFullPath());
+    files.erase(file->fullpath);
     tagfilemap.erase(file->getTagID());
+    file_key.dec(file);
 
-    debugLog("removed file %s\n", file->getFullPath().c_str());
+    debugLog("removed file %s\n", file->fullpath.c_str());
 
     delete file;
 }
@@ -911,17 +1021,8 @@ void Gource::readLog() {
 
 void Gource::processCommit(RCommit& commit, float t) {
 
-    //find user of this commit or create them
-    RUser* user = 0;
-
-    //see if user already exists but if not wait until
-    //we actually use one of their files before adding them
-    std::map<std::string, RUser*>::iterator seen_user = users.find(commit.username);
-    if(seen_user != users.end()) user = seen_user->second;
-
-
     //check user against filters, if found, discard commit
-    if(user == 0 && !gGourceSettings.user_filters.empty()) {
+    if(!gGourceSettings.user_filters.empty()) {
         for(std::vector<Regex*>::iterator ri = gGourceSettings.user_filters.begin(); ri != gGourceSettings.user_filters.end(); ri++) {
             Regex* r = *ri;
 
@@ -935,9 +1036,53 @@ void Gource::processCommit(RCommit& commit, float t) {
     for(std::list<RCommitFile>::iterator it = commit.files.begin(); it != commit.files.end(); it++) {
 
         RCommitFile& cf = *it;
+        RFile* file = 0;     
 
-        RFile* file = 0;
+        //check filename against filters
+        if(!gGourceSettings.file_filters.empty()) {
 
+            bool filtered_filename = false;
+
+            for(std::vector<Regex*>::iterator ri = gGourceSettings.file_filters.begin(); ri != gGourceSettings.file_filters.end(); ri++) {
+                Regex* r = *ri;
+
+                if(r->match(cf.filename)) {
+                    filtered_filename = true;
+                    break;
+                }
+            }
+
+            if(filtered_filename) continue;
+        }
+        
+        //is this a directory (ends in slash)
+        //deleting a directory - find directory: then for each file, remove each file
+                
+        if(!cf.filename.empty() && cf.filename[cf.filename.size()-1] == '/') {
+
+            //ignore unless it is a delete: we cannot 'add' or 'modify' a directory
+            //as its not a physical entity in Gource, only files are.
+
+            if(cf.action != "D") continue;
+            
+            RDirNode* dir = root->findDir(cf.filename);
+
+            if(!dir) continue;
+            
+            //foreach dir files
+            std::list<RFile*> dir_files;
+
+            dir->getFilesRecursive(dir_files);
+            
+            for(std::list<RFile*>::iterator it = dir_files.begin(); it != dir_files.end(); it++) {
+                RFile* file = *it;
+                
+                addFileAction(commit.username, cf.action, file, t);
+            }
+            
+            return;
+        }       
+        
         std::map<std::string, RFile*>::iterator seen_file = files.find(cf.filename);
         if(seen_file != files.end()) file = seen_file->second;
 
@@ -945,27 +1090,9 @@ void Gource::processCommit(RCommit& commit, float t) {
 
             //if we already have max files in circulation
             //we cant add any more
-            if(files.size() >= gGourceSettings.max_files)
+            if(gGourceSettings.max_files > 0 && files.size() >= gGourceSettings.max_files)
                 continue;
 
-            //check filename against filters
-            if(!gGourceSettings.file_filters.empty()) {
-
-                bool filtered_filename = false;
-
-                for(std::vector<Regex*>::iterator ri = gGourceSettings.file_filters.begin(); ri != gGourceSettings.file_filters.end(); ri++) {
-                    Regex* r = *ri;
-
-                    if(r->match(cf.filename)) {
-                        filtered_filename = true;
-                        break;
-                    }
-                }
-
-                if(filtered_filename) continue;
-
-            }
-
             int tagid = tag_seq++;
 
             file = new RFile(cf.filename, cf.colour, vec2f(0.0,0.0), tagid);
@@ -974,70 +1101,78 @@ void Gource::processCommit(RCommit& commit, float t) {
             tagfilemap[tagid]  = file;
 
             root->addFile(file);
-
+            
+            file_key.inc(file);
+            
             while(root->getParent() != 0) {
                 debugLog("parent changed to %s\n", root->getPath().c_str());
                 root = root->getParent();
             }
         }
 
-        //create user if havent yet. do it here to ensure at least one of there files
-        //was added (incase we hit gGourceSettings.max_files)
-
-        if(user == 0) {
-            vec2f pos;
-
-            if(dir_bounds.area() > 0) {
-                pos = dir_bounds.centre();
-            } else {
-                pos = vec2f(0,0);
-            }
-
-            int tagid = tag_seq++;
-
-            user = new RUser(commit.username, pos, tagid);
-
-            users[commit.username] = user;
-            tagusermap[tagid]     = user;
-
-            if(gGourceSettings.highlight_all_users) {
-                user->setHighlighted(true);
-            } else {
-
-                // set the highlighted flag if name matches a highlighted user
-                for(std::vector<std::string>::iterator hi = gGourceSettings.highlight_users.begin(); hi != gGourceSettings.highlight_users.end(); hi++) {
-                    std::string highlight = *hi;
-
-                    if(highlight.size() && user->getName() == highlight) {
-                        user->setHighlighted(true);
-                        break;
-                    }
-                }
-            }
-
-            debugLog("added user %s, tagid = %d\n", commit.username.c_str(), tagid);
-        }
-
-        //create action
-
-        RAction* action = 0;
-
-        int commitNo = commit_seq++;
-
-        if(cf.action == "D") {
-            action = new RemoveAction(user, file, t);
-        } else {
-            if(cf.action == "A") {
-                action = new CreateAction(user, file, t);
-            } else {
-                action = new ModifyAction(user, file, t);
-            }
-        }
-
-        user->addAction(action);
+        addFileAction(commit.username, cf.action, file, t);
     }
 }
 
+void Gource::addFileAction(const std::string& username, const std::string& action, RFile* file, float t) {
+    //create user if havent yet. do it here to ensure at least one of there files
+    //was added (incase we hit gGourceSettings.max_files)
+
+    //find user of this commit or create them
+    RUser* user = 0;
+
+    //see if user already exists
+    std::map<std::string, RUser*>::iterator seen_user = users.find(username);
+    if(seen_user != users.end()) user = seen_user->second;
+
+    if(user == 0) {
+        vec2f pos;
+
+        if(dir_bounds.area() > 0) {
+            pos = dir_bounds.centre();
+        } else {
+            pos = vec2f(0,0);
+        }
+
+        int tagid = tag_seq++;
+
+        user = new RUser(username, pos, tagid);
+
+        users[username] = user;
+        tagusermap[tagid]     = user;
+
+        // set the highlighted flag if name matches a highlighted user
+        for(std::vector<std::string>::iterator hi = gGourceSettings.highlight_users.begin(); hi != gGourceSettings.highlight_users.end(); hi++) {
+            std::string highlight = *hi;
+
+            if(highlight.size() && user->getName() == highlight) {
+                user->setHighlighted(true);
+                break;
+            }
+        }
+
+        debugLog("added user %s, tagid = %d\n", username.c_str(), tagid);
+    }
+
+    //create action
+
+    RAction* userAction = 0;
+
+    int commitNo = commit_seq++;
+
+    if(action == "D") {
+        userAction = new RemoveAction(user, file, t);
+    } else {
+        if(action == "A") {
+            userAction = new CreateAction(user, file, t);
+        } else {
+            userAction = new ModifyAction(user, file, t);
+        }
+    }
+
+    user->addAction(userAction);
+}
+
 void Gource::interactUsers() {
 
 
@@ -1084,14 +1219,18 @@ void Gource::interactUsers() {
 void Gource::updateBounds() {
 
     user_bounds.reset();
+    active_user_bounds.reset();
 
     for(std::map<std::string,RUser*>::iterator it = users.begin(); it!=users.end(); it++) {
         RUser* user = it->second;
 
+        user->updateQuadItemBounds();
+        user_bounds.update(user->quadItemBounds);
+
         if(!user->isIdle()) {
-            user->updateQuadItemBounds();
-            user_bounds.update(user->quadItemBounds);
+            active_user_bounds.update(user->quadItemBounds);
         }
+        
     }
 
     dir_bounds.reset();
@@ -1221,33 +1360,51 @@ void Gource::updateTime(time_t display_time) {
 void Gource::updateCamera(float dt) {
 
     //camera tracking
-    Bounds2D cambounds;
 
     bool auto_rotate = !manual_rotate && !gGourceSettings.disable_auto_rotate;
 
-    if(backgroundSelected) {
-        Bounds2D mousebounds;
-        mousebounds.update(backgroundPos);
+    
+    if(manual_camera) {
 
-        cambounds = mousebounds;
+        if(cursor_move.length2() > 0.0f) {
 
-        auto_rotate = false;
+            float cam_rate = ( -camera.getPos().z ) / ( 5000.0f );
+            
+            vec3f cam_pos = camera.getPos();
+            
+            vec2f cursor_delta = cursor_move * cam_rate * 400.0f * dt;
+           
+            cam_pos.x += cursor_delta.x;
+            cam_pos.y += cursor_delta.y;
+        
+            camera.setPos(cam_pos, true);
+            camera.stop();           
 
-    } else if(track_users && (selectedFile !=0 || selectedUser !=0)) {
-        Bounds2D focusbounds;
+            auto_rotate = false;
 
-        vec3f camerapos = camera.getPos();
+            cursor_move = vec2f(0.0f, 0.0f);
+        }
 
-        if(selectedUser !=0) focusbounds.update(selectedUser->getPos());
-        if(selectedFile !=0) focusbounds.update(selectedFile->getAbsolutePos());
-
-        cambounds = focusbounds;
     } else {
-        if(track_users && idle_time==0) cambounds = user_bounds;
-        else cambounds = dir_bounds;
-    }
 
-    camera.adjust(cambounds);
+        Bounds2D cambounds;
+
+        if(track_users && (selectedFile !=0 || selectedUser !=0)) {
+            Bounds2D focusbounds;
+
+            vec3f camerapos = camera.getPos();
+
+            if(selectedUser !=0) focusbounds.update(selectedUser->getPos());
+            if(selectedFile !=0) focusbounds.update(selectedFile->getAbsolutePos());
+
+            cambounds = focusbounds;
+        } else {
+            if(track_users && idle_time==0) cambounds = active_user_bounds;
+            else cambounds = dir_bounds;
+        }
+
+        camera.adjust(cambounds, !manual_zoom);
+    }
 
     camera.logic(dt);
 
@@ -1277,6 +1434,22 @@ void Gource::updateCamera(float dt) {
     }
 }
 
+//change the string hashing seed and recolour files and users
+void Gource::changeColours() {
+
+    gStringHashSeed = (rand() % 10000) + 1;
+    
+    for(std::map<std::string,RUser*>::iterator it = users.begin(); it != users.end(); it++) {
+        it->second->colourize();
+    }
+
+    for(std::map<std::string,RFile*>::iterator it = files.begin(); it != files.end(); it++) {
+        it->second->colourize();
+    }
+    
+    file_key.colourize();
+}
+
 void Gource::logic(float t, float dt) {
 
     if(draw_loading) return;
@@ -1315,8 +1488,32 @@ void Gource::logic(float t, float dt) {
         }
     }
 
+    file_key.logic(dt);
+
     slider.logic(dt);
 
+    Uint8 *keystate = SDL_GetKeyState(0);
+
+    if(keystate[SDLK_RIGHT]) {
+        cursor_move.x = 10.0;
+        manual_camera = true;
+    }
+
+    if(keystate[SDLK_LEFT]) {
+        cursor_move.x = -10.0;
+        manual_camera = true;
+    }
+
+    if(keystate[SDLK_UP]) {
+        cursor_move.y = -10.0;
+        manual_camera = true;
+    }
+
+    if(keystate[SDLK_DOWN]) {
+        cursor_move.y = 10.0;
+        manual_camera = true;
+    }
+  
     //apply rotation
     if(rotate_angle != 0.0f) {
 
@@ -1336,9 +1533,15 @@ void Gource::logic(float t, float dt) {
         rotate_angle = 0.0f;
     }
 
+    if(recolour) {
+        changeColours();
+        recolour = false;
+    }
+
     //still want to update camera while paused
     if(paused) {
         updateBounds();
+        interactUsers();
         interactDirs();
         updateCamera(dt);
         return;
@@ -1422,73 +1625,57 @@ void Gource::logic(float t, float dt) {
 }
 
 void Gource::mousetrace(Frustum& frustum, float dt) {
-    GLuint	buffer[512];
-	GLint	viewport[4];
-
-	glGetIntegerv(GL_VIEWPORT, viewport);
-	glSelectBuffer(512, buffer);
-
-    glDepthFunc(GL_LEQUAL);
-    glEnable(GL_DEPTH_TEST);
-
-    (void) glRenderMode(GL_SELECT);
-
-    glInitNames();
-    glPushName(0);
-
+    //fprintf(stderr, "start trace\n");
+    //projected mouse pos
     glMatrixMode(GL_PROJECTION);
+    glPushMatrix();
     glLoadIdentity();
-
-    gluPickMatrix((GLdouble) mousepos.x, (GLdouble) (viewport[3]-mousepos.y), 1.0f, 1.0f, viewport);
-    gluPerspective(90.0f, (GLfloat)display.width/(GLfloat)display.height, 0.1f, camera.getZFar());
+    gluPerspective(camera.getFov(), (GLfloat)display.width/(GLfloat)display.height, camera.getZNear(), -camera.getPos().z);
     camera.look();
 
-    glMatrixMode(GL_MODELVIEW);
-    glLoadIdentity();
-
-    for(std::map<std::string,RUser*>::iterator it = users.begin(); it!=users.end(); it++) {
-        it->second->drawSimple(dt);
-    }
-
-    glDisable(GL_TEXTURE_2D);
-    glColor4f(1.0, 1.0, 1.0, 1.0);
-
-    root->drawSimple(frustum, dt);
-
-    glMatrixMode(GL_MODELVIEW);
-
-    mouse_hits = glRenderMode(GL_RENDER);
+    vec2f projected_mouse = display.unproject(mousepos).truncate();
+    glPopMatrix();
 
+    //find user/file under mouse
+    
     RFile* fileSelection = 0;
     RUser* userSelection = 0;
 
-    if (mouse_hits > 0) {
-        int choice   = buffer[3];
-        GLuint depth = buffer[1];
-
-        for (int loop = 1; loop < mouse_hits; loop++) {
-            if (buffer[loop*4+1] < depth) {
-                choice = buffer[loop*4+3];
-                depth  = buffer[loop*4+1];
-            }
+    std::set<QuadItem*> userset;
+    
+    userTree->getItemsAt(userset, projected_mouse);
+    
+    for(std::set<QuadItem*>::iterator it = userset.begin(); it != userset.end(); it++) {
+        RUser* user = (RUser*) *it;
+        if(!user->isFading() && user->quadItemBounds.contains(projected_mouse)) {
+            userSelection = user;
+            break;
         }
-
-        if(choice != 0) {
-            selectionDepth = depth;
-
-            std::map<int, RFile*>::iterator filetest;
-            std::map<int, RUser*>::iterator usertest;
-
-            if((filetest = tagfilemap.find(choice)) != tagfilemap.end()) {
-                fileSelection = filetest->second;
-            }
-            else if((usertest = tagusermap.find(choice)) != tagusermap.end()) {
-                userSelection = usertest->second;
-            }
-		}
     }
 
-    glDisable(GL_DEPTH_TEST);
+    if(!userSelection) {
+    
+        std::set<QuadItem*> dirset;
+
+        dirNodeTree->getItemsAt(dirset, projected_mouse);
+                
+        for(std::set<QuadItem*>::iterator it = dirset.begin(); it != dirset.end(); it++) {
+
+            RDirNode* dir = (RDirNode*) *it;
+            
+            const std::list<RFile*>* files = dir->getFiles(); 
+           
+            for(std::list<RFile*>::const_iterator fi = files->begin(); fi != files->end(); fi++) {
+            
+                RFile* file = *fi;                              
+                
+                if(!file->isHidden() && file->overlaps(projected_mouse)) {
+                    fileSelection = file;
+                    break;
+                }
+            }
+        }
+    }
 
     // is over a file
     if(fileSelection != 0) {
@@ -1531,12 +1718,19 @@ void Gource::mousetrace(Frustum& frustum, float dt) {
     }
 
     if(mouseclicked) {
-        if(hoverUser!=0) selectUser(hoverUser);
-        else if(hoverFile!=0) selectFile(hoverFile);
-        else {
+
+        if(hoverUser!=0) {
+            camera.lockOn(false);
+            selectUser(hoverUser);
+        } else if(hoverFile!=0) {
+            camera.lockOn(false);
+            selectFile(hoverFile);
+        } else {
             selectBackground();
         }
     }
+    
+    //fprintf(stderr, "end trace\n");   
 }
 
 void Gource::loadingScreen() {
@@ -1764,6 +1958,15 @@ void Gource::draw(float t, float dt) {
 
     if(!gGourceSettings.hide_mouse && cursor.isVisible()) {
         mousetrace(frustum,dt);
+    } else {
+        if(hoverUser) {
+            hoverUser->setMouseOver(false);
+            hoverUser = 0;
+        }
+        if(hoverFile) {
+            hoverFile->setMouseOver(false);
+            hoverFile = 0;
+        }
     }
 
     trace_time = SDL_GetTicks() - trace_time;
@@ -1798,7 +2001,6 @@ void Gource::draw(float t, float dt) {
 
     name_calc_time = SDL_GetTicks() - name_calc_time;
 
-
     name_draw_time = SDL_GetTicks();
 
     //switch to 2D, preserve current state
@@ -1831,19 +2033,25 @@ void Gource::draw(float t, float dt) {
         glLineWidth(2.0);
         glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
 
-        track_users ? user_bounds.draw() : dir_bounds.draw();
+        track_users ? active_user_bounds.draw() : dir_bounds.draw();
     }
 
     if(gGourceQuadTreeDebug) {
         glDisable(GL_TEXTURE_2D);
-        glColor4f(0.0f, 1.0f, 0.0f, 1.0f);
 
         glLineWidth(1.0);
+
+        glColor4f(0.0f, 1.0f, 0.0f, 1.0f);
+
         dirNodeTree->outline();
 
         glColor4f(0.0f, 1.0f, 1.0f, 1.0f);
 
         userTree->outline();
+
+        glColor4f(0.0f, 1.0f, 0.5f, 1.0f);
+
+        userTree->outlineItems();
     }
 
     glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
@@ -1931,15 +2139,50 @@ void Gource::draw(float t, float dt) {
          fontmedium.draw(1, 3, message);
     }
 
-    // end text
+    //file key
+    file_key.draw();
+    file_key.setShow(gGourceSettings.show_key);
 
-    glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
+    //slider
+    if(canSeek()) {
+        slider.draw(dt);
+    }
+
+    //text box   
+    if(hoverFile && hoverFile != selectedFile) {
+        
+        std::string display_path = hoverFile->path;
+        display_path.erase(0,1);
+        
+        textbox.setText(hoverFile->getName());
+        if(display_path.size()) textbox.addLine(display_path);
+        textbox.setColour(hoverFile->getColour());
+
+        textbox.setPos(mousepos, true);
+        textbox.draw();
+    } else if(hoverUser && hoverUser != selectedUser) {
+
+        textbox.setText(hoverUser->getName());
+        textbox.setColour(hoverUser->getColour());
+        
+        textbox.setPos(mousepos, true);
+        textbox.draw();
+    }
+
+    //debug info
 
     if(debug) {
+        glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
+
+        glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+        glEnable(GL_BLEND);
+        glEnable(GL_TEXTURE_2D);
+
+        
         font.print(1,20, "FPS: %.2f", fps);
         font.print(1,40,"Days Per Second: %.2f",
             gGourceSettings.days_per_second);
-        font.print(1,60,"Time Scale: %.2f", time_scale);
+        font.print(1,60,"Time Scale: %.2f", gGourceSettings.time_scale);
         font.print(1,80,"Users: %d", users.size());
         font.print(1,100,"Files: %d", files.size());
         font.print(1,120,"Dirs: %d",  gGourceDirMap.size());
@@ -1954,25 +2197,21 @@ void Gource::draw(float t, float dt) {
         font.print(1,280,"Draw Time: %u ms (Names: Calc Time = %u ms, Draw Time = %u ms)", SDL_GetTicks() - draw_time, name_calc_time, name_draw_time);
         font.print(1,300,"File Inner Loops: %d", gGourceFileInnerLoops);
         font.print(1,320,"User Inner Loops: %d", gGourceUserInnerLoops);
-        font.print(1,340,"Dir Inner Loops: %d (QTree items = %d, nodes = %d)", gGourceDirNodeInnerLoops,
-            dirNodeTree->item_count, dirNodeTree->node_count);
+        font.print(1,340,"Dir Inner Loops: %d (QTree items = %d, nodes = %d, max node depth = %d)", gGourceDirNodeInnerLoops,
+            dirNodeTree->item_count, dirNodeTree->node_count, dirNodeTree->max_node_depth);
         font.print(1,360,"Dir Bounds Ratio: %.2f, %.5f", dir_bounds.width() / dir_bounds.height(), rotation_remaining_angle);
+        font.print(1,380,"String Hash Seed: %d", gStringHashSeed);
 
         if(selectedUser != 0) {
 
         }
 
         if(selectedFile != 0) {
-            font.print(1,360,"%s: %d files (%d visible)", selectedFile->getDir()->getPath().c_str(),
+            font.print(1,400,"%s: %d files (%d visible)", selectedFile->getDir()->getPath().c_str(),
                     selectedFile->getDir()->fileCount(), selectedFile->getDir()->visibleFileCount());
         }
-
     }
 
-    glDisable(GL_TEXTURE_2D);
-
-    if(canSeek()) slider.draw(dt);
-
     mousemoved=false;
     mouseclicked=false;
 }
