@@ -41,17 +41,20 @@
 #include "hg.h"
 #include "bzr.h"
 #include "gitraw.h"
+#include "cvs2cl.h"
 #include "cvs-exp.h"
 #include "custom.h"
 #include "apache.h"
+#include "svn.h"
 
 #include "slider.h"
-
+#include "textbox.h"
 #include "action.h"
 #include "file.h"
 #include "user.h"
 #include "dirnode.h"
 #include "zoomcamera.h"
+#include "key.h"
 
 class Gource : public SDLApp {
     std::string logfile;
@@ -62,9 +65,14 @@ class Gource : public SDLApp {
     PositionSlider slider;
     ZoomCamera camera;
 
+    FileKey file_key;
+
     bool debug, trace_debug;
 
+    bool manual_zoom;
     bool manual_rotate;
+    bool manual_camera;
+
     float rotation_remaining_angle;
 
     MouseCursor cursor;
@@ -74,15 +82,19 @@ class Gource : public SDLApp {
     bool mouseclicked;
     bool mousedragged;
 
+    
+    vec2f cursor_move;
+    
+    bool recolour;
+    
+    bool use_selection_bounds;
+    Bounds2D selection_bounds;
+    
     float rotate_angle;
 
     vec2f mousepos;
 
-    vec2f backgroundPos;
-    bool backgroundSelected;
-
     float last_percent;
-    float time_scale;
 
     bool stop_position_reached;
 
@@ -108,6 +120,8 @@ class Gource : public SDLApp {
     TextureResource* logotex;
     TextureResource* backgroundtex;
 
+    TextBox textbox;
+    
     FXFont font, fontlarge, fontmedium;
 
     bool first_read;
@@ -140,6 +154,7 @@ class Gource : public SDLApp {
 
     Bounds2D dir_bounds;
     Bounds2D user_bounds;
+    Bounds2D active_user_bounds;
 
     std::deque<RCommit> commitqueue;
     std::map<std::string, RUser*> users;
@@ -166,13 +181,15 @@ class Gource : public SDLApp {
     void selectNextUser();
 
     void readLog();
+
     void processCommit(RCommit& commit, float t);
+    void addFileAction(const std::string& username, const std::string& action, RFile* file, float t);
 
     std::string dateAtPosition(float percent);
 
     void toggleCameraMode();
 
-    RCommitLog* determineFormat(const std::string& logfile);
+    static RCommitLog* determineFormat(const std::string& logfile);
 
     void updateCamera(float dt);
 
@@ -200,10 +217,14 @@ class Gource : public SDLApp {
     void drawBloom(Frustum &frustum, float dt);
 
     void screenshot();
+
+    void changeColours();
 public:
     Gource(FrameExporter* frameExporter = 0);
     ~Gource();
 
+    static void writeCustomLog(const std::string& logfile, const std::string& output_file);
+
     void setCameraMode(const std::string& mode);
     void setCameraMode(bool track_users);
     void setFrameExporter(FrameExporter* exporter, int video_framerate);
