@@ -26,7 +26,7 @@ void GourceSettings::help(bool extended_help) {
     SDLAppCreateWindowsConsole();
 
     //resize window to fit help message
-    SDLAppResizeWindowsConsole(770);
+    SDLAppResizeWindowsConsole(800);
 #endif
 
     printf("Gource v%s\n", GOURCE_VERSION);
@@ -50,21 +50,22 @@ void GourceSettings::help(bool extended_help) {
     printf("      --disable-auto-skip          Disable auto skip\n");
     printf("  -s, --seconds-per-day SECONDS    Speed in seconds per day (default: 10)\n");
     printf("      --realtime                   Realtime playback speed\n");
+    printf("  -c, --time-scale SCALE           Change simuation time scale (default: 1.0)\n");
     printf("  -e, --elasticity FLOAT           Elasticity of nodes\n\n");
 
+    printf("  --key                            Show file extension key\n\n");
+
     printf("  --user-image-dir DIRECTORY       Dir containing images to use as avatars\n");
     printf("  --default-user-image IMAGE       Default user image file\n");
     printf("  --colour-images                  Colourize user images\n\n");
 
-    printf("  -i, --file-idle-time SECONDS     Time files remain idle (default: 60)\n");
-    printf("      --file-extensions            Show filename extensions only\n\n");
+    printf("  -i, --file-idle-time SECONDS     Time files remain idle (default: 60)\n\n");
 
     printf("  --max-files NUMBER       Max number of active files (default: 1000)\n");
     printf("  --max-file-lag SECONDS   Max time files of a commit can take to appear\n\n");
 
-    printf("  --log-command VCS        Show the log command used by gource (git,cvs,hg,bzr)\n");
-    printf("  --log-format  VCS        Specify format of the log (git,cvs,hg,bzr,custom)\n");
-    printf("  --git-branch             Get the git log of a particular branch\n\n");
+    printf("  --log-command VCS        Show the VCS log command (git,svn,hg,bzr,cvs2cl)\n");
+    printf("  --log-format  VCS        Specify the log format (git,svn,hg,bzr,cvs2cl,custom)\n\n");
 
     printf("  --load-config CONF_FILE  Load a config file\n");
     printf("  --save-config CONF_FILE  Save a config file with the current options\n\n");
@@ -75,6 +76,8 @@ void GourceSettings::help(bool extended_help) {
 if(extended_help) {
     printf("Extended Options:\n\n");
 
+    printf("  --output-custom-log FILE  Output a custom format log file ('-' for STDOUT).\n\n");
+
     printf("  -b, --background-colour  FFFFFF    Background colour in hex\n");
     printf("      --background-image   IMAGE     Set a background image\n\n");
 
@@ -92,6 +95,10 @@ if(extended_help) {
     printf("  --font-size SIZE         Font size\n");
     printf("  --font-colour FFFFFF     Font colour in hex\n\n");
 
+    printf("  --file-extensions        Show filename extensions only\n\n");
+
+    printf("  --git-branch             Get the git log of a particular branch\n\n");
+
     printf("  --hide DISPLAY_ELEMENT   bloom,date,dirnames,files,filenames,mouse,progress,\n");
     printf("                           tree,users,usernames\n\n");
 
@@ -112,7 +119,10 @@ if(extended_help) {
     printf("  --follow-user USER       Camera will automatically follow this user\n");
     printf("  --highlight-user USER    Highlight the names of a particular user\n");
     printf("  --highlight-users        Highlight the names of all users\n\n");
-    printf("  --highlight-dirs         Highlight the names of all directories\n\n");
+    printf("  --highlight-dirs         Highlight the names of all directories\n");
+    printf("  --highlight-colour       Font colour for highlighted text\n\n");
+
+    printf("  --hash-seed SEED         Change the seed of hash function.\n\n");
 
     printf("  --path PATH\n\n");
 }
@@ -152,6 +162,7 @@ GourceSettings::GourceSettings() {
     arg_aliases["?"] = "help";
     arg_aliases["H"] = "extended-help";
     arg_aliases["b"] = "background-colour";
+    arg_aliases["c"] = "time-scale";
     arg_aliases["background"] = "background-colour";
     arg_aliases["disable-bloom"]    = "hide-bloom";
     arg_aliases["disable-progress"] = "hide-progress";
@@ -163,10 +174,13 @@ GourceSettings::GourceSettings() {
     conf_sections["log-command"]     = "command-line";
     conf_sections["git-log-command"] = "command-line";
     conf_sections["cvs-exp-command"] = "command-line";
+    conf_sections["cvs2cl-command"]  = "command-line";
     conf_sections["hg-log-command"]  = "command-line";
     conf_sections["bzr-log-command"] = "command-line";
+    conf_sections["svn-log-command"] = "command-line";
     conf_sections["load-config"]     = "command-line";
     conf_sections["save-config"]     = "command-line";
+    conf_sections["output-custom-log"] = "command-line";
 
     //boolean args
     arg_types["help"]            = "bool";
@@ -188,14 +202,17 @@ GourceSettings::GourceSettings() {
     arg_types["hide-bloom"]      = "bool";
     arg_types["hide-mouse"]      = "bool";
     arg_types["highlight-all-users"] = "bool";
-    arg_types["highlight-dirs"] = "bool";
+    arg_types["highlight-dirs"]  = "bool";
     arg_types["file-extensions"] = "bool";
+    arg_types["key"]             = "bool";
 
     arg_types["disable-auto-rotate"] = "bool";
     arg_types["disable-auto-skip"]   = "bool";
 
     arg_types["git-log-command"]= "bool";
     arg_types["cvs-exp-command"]= "bool";
+    arg_types["cvs2cl-command"] = "bool";
+    arg_types["svn-log-command"]= "bool";
     arg_types["hg-log-command"] = "bool";
     arg_types["bzr-log-command"]= "bool";
 
@@ -208,9 +225,11 @@ GourceSettings::GourceSettings() {
     arg_types["max-user-speed"]    = "float";
     arg_types["user-friction"]     = "float";
     arg_types["padding"]           = "float";
+    arg_types["time-scale"]        = "float";
 
     arg_types["max-files"] = "int";
     arg_types["font-size"] = "int";
+    arg_types["hash-seed"] = "int";
 
     arg_types["user-filter"]    = "multi-value";
     arg_types["file-filter"]    = "multi-value";
@@ -223,6 +242,7 @@ GourceSettings::GourceSettings() {
     arg_types["log-command"]        = "string";
     arg_types["load-config"]        = "string";
     arg_types["save-config"]        = "string";
+    arg_types["output-custom-log"]  = "string";
     arg_types["path"]               = "string";
     arg_types["log-command"]        = "string";
     arg_types["background-colour"]  = "string";
@@ -241,6 +261,7 @@ GourceSettings::GourceSettings() {
     arg_types["camera-mode"]        = "string";
     arg_types["title"]              = "string";
     arg_types["font-colour"]        = "string";
+    arg_types["highlight-colour"]   = "string";
 }
 
 void GourceSettings::setGourceDefaults() {
@@ -265,11 +286,14 @@ void GourceSettings::setGourceDefaults() {
     stop_at_end    = false;
     dont_stop      = false;
 
+    show_key = false;
+
     disable_auto_rotate = false;
 
     auto_skip_seconds = 3.0f;
     days_per_second   = 0.1f; // TODO: check this is right
     file_idle_time    = 60.0f;
+    time_scale        = 1.0f;
 
     loop = false;
 
@@ -297,6 +321,7 @@ void GourceSettings::setGourceDefaults() {
 
     font_size = 16;
     font_colour = vec3f(1.0f, 1.0f, 1.0f);
+    highlight_colour = vec3f(1.0f, 1.0f, 0.3f);
 
     elasticity = 0.0f;
 
@@ -318,6 +343,8 @@ void GourceSettings::setGourceDefaults() {
     highlight_all_users = false;
     highlight_dirs = false;
 
+    gStringHashSeed = 31;
+
     //delete file filters
     for(std::vector<Regex*>::iterator it = file_filters.begin(); it != file_filters.end(); it++) {
         delete (*it);
@@ -362,10 +389,22 @@ void GourceSettings::commandLineOption(const std::string& name, const std::strin
         SDLAppInfo(gGourceGitLogCommand);
     }
 
-    if(name == "cvs-exp-command" || log_command == "cvs") {
+    if(name == "cvs-exp-command" || log_command == "cvs-exp") {
         SDLAppInfo(gGourceCvsExpLogCommand);
     }
 
+    if(log_command == "cvs") {
+        throw ConfFileException("please use either 'cvs2cl' or 'cvs-exp'", "", 0);
+    }
+
+    if(name == "cvs2cl-command" || log_command == "cvs2cl") {
+        SDLAppInfo(gGourceCVS2CLLogCommand);
+    }
+
+    if(name == "svn-log-command" || log_command == "svn") {
+        SDLAppInfo(gGourceSVNLogCommand);
+    }
+
     if(name == "hg-log-command" || log_command == "hg") {
         std::string command = gGourceMercurialCommand();
         SDLAppInfo(command);
@@ -376,11 +415,19 @@ void GourceSettings::commandLineOption(const std::string& name, const std::strin
         SDLAppInfo(command);
     }
 
+    if(name == "output-custom-log" && value.size() > 0) {
+        output_custom_filename = value;
+        return;
+    }
+
     std::string invalid_error = std::string("invalid ") + name + std::string(" value");
     throw ConfFileException(invalid_error, "", 0);
 }
 
 
+#ifdef __APPLE__
+#include <CoreFoundation/CoreFoundation.h>
+#endif
 void GourceSettings::importGourceSettings(ConfFile& conffile, ConfSection* gource_settings) {
 
     setGourceDefaults();
@@ -524,8 +571,14 @@ void GourceSettings::importGourceSettings(ConfFile& conffile, ConfSection* gourc
 
         log_format = entry->getString();
 
+        if(log_format == "cvs") {
+            conffile.entryException(entry, "please use either 'cvs2cl' or 'cvs-exp'");
+        }
+
         if(   log_format != "git"
-           && log_format != "cvs"
+           && log_format != "cvs-exp"
+           && log_format != "cvs2cl"
+           && log_format != "svn"
            && log_format != "custom"
            && log_format != "hg"
            && log_format != "bzr"
@@ -574,6 +627,18 @@ void GourceSettings::importGourceSettings(ConfFile& conffile, ConfSection* gourc
                 std::string image_path = gGourceSettings.user_image_dir + dirfile;
                 std::string name       = dirfile.substr(0,extpos);
 
+#ifdef __APPLE__
+                CFMutableStringRef help = CFStringCreateMutable(kCFAllocatorDefault, 0);
+                CFStringAppendCString(help, name.c_str(), kCFStringEncodingUTF8);
+                CFStringNormalize(help, kCFStringNormalizationFormC);
+                char data[4096];
+                CFStringGetCString(help,
+                                   data,
+                                   sizeof(data),
+                                   kCFStringEncodingUTF8);
+                name = data;
+#endif
+
                 debugLog("%s => %s\n", name.c_str(), image_path.c_str());
 
                 user_image_map[name] = image_path;
@@ -627,6 +692,13 @@ void GourceSettings::importGourceSettings(ConfFile& conffile, ConfSection* gourc
         }
     }
 
+    if((entry = gource_settings->getEntry("hash-seed")) != 0) {
+
+        if(!entry->hasValue()) conffile.entryException(entry, "specify hash seed (integer)");
+
+        gStringHashSeed = entry->getInt();
+    }
+
     if((entry = gource_settings->getEntry("font-colour")) != 0) {
 
         if(!entry->hasValue()) conffile.entryException(entry, "specify font colour (FFFFFF)");
@@ -663,6 +735,24 @@ void GourceSettings::importGourceSettings(ConfFile& conffile, ConfSection* gourc
         }
     }
 
+    if((entry = gource_settings->getEntry("highlight-colour")) != 0) {
+
+        if(!entry->hasValue()) conffile.entryException(entry, "specify highlight colour (FFFFFF)");
+
+        int r,g,b;
+
+        std::string colstring = entry->getString();
+
+        if(entry->isVec3()) {
+            highlight_colour = entry->getVec3();
+        } else if(colstring.size()==6 && sscanf(colstring.c_str(), "%02x%02x%02x", &r, &g, &b) == 3) {
+            highlight_colour = vec3f(r,g,b);
+            highlight_colour /= 255.0f;
+        } else {
+            conffile.invalidValueException(entry);
+        }
+    }
+
     if((entry = gource_settings->getEntry("background-image")) != 0) {
 
         if(!entry->hasValue()) conffile.entryException(entry, "specify background image (image path)");
@@ -753,6 +843,17 @@ void GourceSettings::importGourceSettings(ConfFile& conffile, ConfSection* gourc
         }
     }
 
+    if((entry = gource_settings->getEntry("time-scale")) != 0) {
+
+        if(!entry->hasValue())
+            conffile.entryException(entry, "specify time-scale (scale)");
+
+        time_scale = entry->getFloat();
+
+        if(time_scale <= 0.0f || time_scale > 4.0f) {
+            conffile.entryException(entry, "time-scale outside of range 0.0 - 4.0");
+        }
+    }
 
     if((entry = gource_settings->getEntry("start-position")) != 0) {
 
@@ -792,6 +893,10 @@ void GourceSettings::importGourceSettings(ConfFile& conffile, ConfSection* gourc
         }
     }
 
+    if(gource_settings->getBool("key")) {
+        show_key = true;
+    }
+
     if(gource_settings->getBool("realtime")) {
         days_per_second = 1.0 / 86400.0;
     }
@@ -815,7 +920,7 @@ void GourceSettings::importGourceSettings(ConfFile& conffile, ConfSection* gourc
 
         max_files = entry->getInt();
 
-        if(max_files<1) {
+        if(max_files<0 || max_files == 0 && entry->getString() != "0") {
             conffile.invalidValueException(entry);
         }
     }
@@ -990,7 +1095,18 @@ void GourceSettings::importGourceSettings(ConfFile& conffile, ConfSection* gourc
             throw ConfFileException("log-format required when reading from STDIN", "", 0);
         }
 
+#ifdef _WIN32
+        DWORD available_bytes;
+        HANDLE stdin_handle = GetStdHandle(STD_INPUT_HANDLE);
+
+        while(PeekNamedPipe(stdin_handle, 0, 0, 0,
+            &available_bytes, 0) && available_bytes==0 && !std::cin.fail()) {
+            SDL_Delay(100);
+        }
+#else
         while(std::cin.peek() == EOF && !std::cin.fail()) SDL_Delay(100);
+#endif
+
         std::cin.clear();
     }
 
