@@ -18,7 +18,7 @@
 #ifndef GOURCE_SETTINGS_H
 #define GOURCE_SETTINGS_H
 
-#define GOURCE_VERSION "0.28"
+#define GOURCE_VERSION "0.32"
 
 #include <dirent.h>
 
@@ -26,6 +26,8 @@
 #include "git.h"
 #include "bzr.h"
 #include "cvs-exp.h"
+#include "cvs2cl.h"
+#include "svn.h"
 
 #include "core/settings.h"
 #include "core/regex.h"
@@ -49,6 +51,8 @@ public:
 
     bool disable_auto_rotate;
 
+    bool show_key;
+    
     std::string load_config;
     std::string save_config;
     std::string path;
@@ -106,14 +110,19 @@ public:
     float user_idle_time;
     float user_friction;
     float user_scale;
+    float time_scale;
 
     bool highlight_dirs;
     bool highlight_all_users;
+    vec3f highlight_colour;
+   
     std::vector<std::string> highlight_users;
     std::vector<std::string> follow_users;
     std::vector<Regex*> file_filters;
     std::vector<Regex*> user_filters;
     bool file_extensions;
+    
+    std::string output_custom_filename;
 
     TextureResource* file_graphic;
 
